@@ -5,9 +5,10 @@ use Log::Handler;
 
 my $log = Log::Handler->new();
 
-$log->config(filename => 'example-1.cfg');
+$log->config(config => 'example-1.cfg');
+
+$log->debug('debug message');
 $log->info('info message');
 $log->warn('warn message');
 $log->crit('crit message');
 $log->fatal('fatal message');
-$log->debug('debug message');
